// glmfcView.cpp : implementation of the CglmfcView class
//

#include "stdafx.h"
#include "glmfc.h"

#include "glmfcDoc.h"
#include "glmfcView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CglmfcView

IMPLEMENT_DYNCREATE(CglmfcView, CView)

BEGIN_MESSAGE_MAP(CglmfcView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
END_MESSAGE_MAP()

// CglmfcView construction/destruction

CglmfcView::CglmfcView()
{
  m_hrc=0;
  m_pDC =0;
}

CglmfcView::~CglmfcView()
{
}

BOOL CglmfcView::PreCreateWindow(CREATESTRUCT& cs)
{
  // An OpenGL window must be created with the following flags and must not
	// include CS_PARENTDC for the class style. Refer to SetPixelFormat
	// documentation in the "Comments" section for further information.
   cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN | CS_OWNDC;
	if(cs.style&CS_PARENTDC)
	{
		AfxMessageBox("Win OpenGL Internal Error!");
		return FALSE;
	}

	return CView::PreCreateWindow(cs);
}

// CglmfcView drawing

void CglmfcView::OnDraw(CDC* /*pDC*/)
{
	CglmfcDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
  TRACE("OnDraw\n");
SetGL();
	glClear(GL_COLOR_BUFFER_BIT);
	GLfloat triangle[3][2]={{-1.0f,-1.0f},{1.0f,-1.0f},{ 0.0f, 1.0f}};
	GLfloat p[2]={0.0, 0.0f};
	int i, j;
   glClear (GL_COLOR_BUFFER_BIT);
	glBegin(GL_POINTS);
	for(j=0;j<20000;j++)	{
		glVertex2f(p[0],p[1]);
		i=rand()%3;
		p[0]=(p[0]+triangle[i][0])/2.0f;
		p[1]=(p[1]+triangle[i][1])/2.0f;
	}
	glEnd();
	
	glFlush();
	SwapBuffers(m_pDC->GetSafeHdc());
}

BOOL CglmfcView::SetupPixelFormat( CDC * pDC )
{
    static PIXELFORMATDESCRIPTOR pfd = 
	 {
		sizeof(PIXELFORMATDESCRIPTOR),  //  size of this pfd
			1,                     // version number
			PFD_DRAW_TO_WINDOW |   // support window
			PFD_SUPPORT_OPENGL |   // support OpenGL
			PFD_DOUBLEBUFFER,      // double buffered
			PFD_TYPE_RGBA,         // RGBA type 
			24,                    // 24-bit color depth 
			0, 0, 0, 0, 0, 0,      // color bits ignored
			0,                     // no alpha buffer
			0,                     // shift bit ignored
			0,                     // no accumulation buffer
			0, 0, 0, 0,            // accum bits ignored
			32,                    // 32-bit z-buffer
			0,                     // no stencil buffer
			0,                     // no auxiliary buffer
			PFD_MAIN_PLANE,        // main layer
			0,                     // reserved
			0, 0, 0                // layer masks ignored
	 };
    int pixelformat;

    if ( (pixelformat = ChoosePixelFormat(pDC->GetSafeHdc(), &pfd)) == 0 )
    {
        AfxMessageBox("ChoosePixelFormat failed");
	    return FALSE;
    }

    if (SetPixelFormat(pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
    {
        AfxMessageBox("SetPixelFormat failed");
	    return FALSE;
    }

    return TRUE;
}

// CglmfcView printing

BOOL CglmfcView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CglmfcView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CglmfcView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CglmfcView diagnostics

#ifdef _DEBUG
void CglmfcView::AssertValid() const
{
	CView::AssertValid();
}

void CglmfcView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CglmfcDoc* CglmfcView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CglmfcDoc)));
	return (CglmfcDoc*)m_pDocument;
}
#endif //_DEBUG


// CglmfcView message handlers

BOOL CglmfcView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
	if( ! CView::Create(lpszClassName, lpszWindowName, dwStyle, 
		rect, pParentWnd, nID, pContext)) 	return false;

  m_pDC = new CClientDC(this);
  ASSERT(m_pDC != NULL);

  // Setta il pixel format
  if (!SetupPixelFormat(m_pDC)) return FALSE;
  // Crea e setta il contesto OPENGL
  m_hrc = wglCreateContext(m_pDC->GetSafeHdc());
  if(m_hrc==NULL){
		AfxMessageBox("OpenGL contest fail");
		return FALSE;
	}
	return TRUE;
}

void CglmfcView::OnDestroy()
{
	CView::OnDestroy();
	SetGL();
	glFinish();
	wglMakeCurrent(NULL,  NULL);
  if (m_hrc) ::wglDeleteContext(m_hrc);
  if (m_pDC) delete m_pDC;

}

BOOL CglmfcView::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}

void CglmfcView::OnSize(UINT nType, int cx, int cy)
{
	TRACE("OnSize\n ");
	if(SetGL()){
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)cx/(float)cy;
	glOrtho(-1,1,-ratio,ratio,-1,1);
	glViewport (0, 0, (GLsizei) cx, (GLsizei) cy); 
	glMatrixMode (GL_MODELVIEW);    /*  back to modelview matrix    */
	}
}
